@@ -196,6 +196,7 @@ def global_best(steps: Dict, rounds: Tuple, args: Dict, n_particles: int, iters:
         worker.start()
 
     r_below = 0
+    early_exit = False
     start_time = datetime.datetime.now()
     for r in range(max_rounds):
         no_improvement = np.full(len(steps), True)
@@ -204,6 +205,7 @@ def global_best(steps: Dict, rounds: Tuple, args: Dict, n_particles: int, iters:
             # check if forced exit.
             if path.exists("stop"):
                 print('\n>>>>> stop file found, exit now.')
+                early_exit = True
                 break
 
             param_names, bounds, objfunc = utils.get_step_info(steps, s)
@@ -237,7 +239,8 @@ def global_best(steps: Dict, rounds: Tuple, args: Dict, n_particles: int, iters:
                                               iters=step.get('iters', iters),
                                               **args)
             if cost is None:
-                exit(1)
+                early_exit = True
+                break
 
             # capture the best cost
             # if cost < best_cost[s] and np.abs(cost - best_cost[s]) > rtol:
@@ -260,6 +263,10 @@ def global_best(steps: Dict, rounds: Tuple, args: Dict, n_particles: int, iters:
 
             # print(json.dumps(steps, sort_keys=False, indent=2))
 
+        if early_exit:
+            step_trace['exit'] = '1'
+            break
+
         round_cost = np.sum(best_cost)
 
         # if no improvement in all steps, break out of rounds prematurely
@@ -284,7 +291,7 @@ def global_best(steps: Dict, rounds: Tuple, args: Dict, n_particles: int, iters:
 
         if (r + 1 >= min_rounds) and 0 <= rel_round_tol < rtol:
             r_below += 1
-            if r_below == rtol_iter:
+            if r_below >= rtol_iter:
                 break
         else:
             # reset
