@@ -22,12 +22,14 @@ import inspect
 import logging
 
 import numpy as np
+import math
+from copy import copy
 
 from ..utils.reporter import Reporter
 
 
 class HandlerMixin(object):
-    """ A HandlerMixing class
+    """A HandlerMixing class
 
     This class offers some basic functionality for the Handlers.
     """
@@ -60,7 +62,7 @@ class HandlerMixin(object):
 
 class BoundaryHandler(HandlerMixin):
     def __init__(self, strategy):
-        """ A BoundaryHandler class
+        """A BoundaryHandler class
 
         This class offers a way to handle boundary conditions. It contains
         methods to repair particle positions outside of the defined boundaries.
@@ -140,7 +142,7 @@ class BoundaryHandler(HandlerMixin):
             return new_position
 
     def nearest(self, position, bounds, **kwargs):
-        """Set position to nearest bound
+        r"""Set position to nearest bound
 
         This method resets particles that exceed the bounds to the nearest
         available boundary. For every axis on which the coordiantes of the particle
@@ -165,7 +167,7 @@ class BoundaryHandler(HandlerMixin):
         return new_pos
 
     def reflective(self, position, bounds, **kwargs):
-        """Reflect the particle at the boundary
+        r"""Reflect the particle at the boundary
 
         This method reflects the particles that exceed the bounds at the
         respective boundary. This means that the amount that the component
@@ -211,7 +213,7 @@ class BoundaryHandler(HandlerMixin):
         return new_pos
 
     def shrink(self, position, bounds, **kwargs):
-        """Set the particle to the boundary
+        r"""Set the particle to the boundary
 
         This method resets particles that exceed the bounds to the intersection
         of its previous velocity and the boundary. This can be imagined as shrinking
@@ -391,7 +393,7 @@ class BoundaryHandler(HandlerMixin):
 
 class VelocityHandler(HandlerMixin):
     def __init__(self, strategy):
-        """ A VelocityHandler class
+        """A VelocityHandler class
 
         This class offers a way to handle velocities. It contains
         methods to repair the velocities of particles that exceeded the
@@ -467,6 +469,7 @@ class VelocityHandler(HandlerMixin):
         r"""Adjust the velocity to the new position
 
         The velocity is adjusted such that the following equation holds:
+
         .. math::
 
                 \mathbf{v_{i,t}} = \mathbf{x_{i,t}} - \mathbf{x_{i,t-1}}
@@ -492,12 +495,13 @@ class VelocityHandler(HandlerMixin):
             return new_vel
 
     def invert(self, velocity, clamp=None, **kwargs):
-        """Invert the velocity if the particle is out of bounds
+        r"""Invert the velocity if the particle is out of bounds
 
         The velocity is inverted and shrinked. The shrinking is determined by the
         kwarg :code:`z`. The default shrinking factor is :code:`0.5`. For all
         velocities whose particles are out of bounds the following equation is
         applied:
+
         .. math::
 
             \mathbf{v_{i,t}} = -z\mathbf{v_{i,t}}
@@ -538,3 +542,221 @@ class VelocityHandler(HandlerMixin):
             raise
         else:
             return new_vel
+
+
+class OptionsHandler(HandlerMixin):
+    def __init__(self, strategy):
+        """An OptionsHandler class
+
+        This class offers a way to handle options. It contains
+        methods to vary the options at runtime.
+        Following strategies are available for the handling:
+
+        * exp_decay:
+            Decreases the parameter exponentially between limits.
+
+        * lin_variation:
+            Decreases/increases the parameter linearly between limits.
+
+        * random:
+            takes a uniform random value between (0.5,1)
+
+        * nonlin_mod:
+            Decreases/increases the parameter between limits according to a nonlinear modulation index .
+
+        * adaptive:
+             Updating coefficients over the iteration.
+
+        The OptionsHandler can be called as a function to use the strategy
+        that is passed at initialization to account for time-varying coefficients. An example
+        for the usage:
+
+        .. code-block :: python
+
+            from pyswarms.backend import operators as op
+            from pyswarms.backend.handlers import OptionsHandler
+
+
+            oh = OptionsHandler(strategy={ "w":"exp_decay", "c1":"nonlin_mod","c2":"lin_variation"})
+
+            for i in range(iters):
+                # some initial stuff
+                new_options = oh(default_options, iternow=i, itermax=iters, end_opts={"c1":0.5, "c2":2.5, "w":0.4})
+                # more updates using new_options
+
+        By passing the handler, the :func:`compute_position()` function now has
+        the ability to reset the particles by calling the :code:`BoundaryHandler`
+        inside.
+
+        Attributes
+        ----------
+        strategy : str
+            The strategy to use. To see all available strategies,
+            call :code:`OptionsHandler.strategies`
+        """
+        self.strategy = strategy
+        self.strategies = self._get_all_strategies()
+        self.rep = Reporter(logger=logging.getLogger(__name__))
+
+    def __call__(self, start_opts, **kwargs):
+        try:
+            if not self.strategy:
+                return start_opts
+            return_opts = copy(start_opts)
+            for opt in start_opts:
+                if opt in self.strategy:
+                    return_opts[opt] = self.strategies[self.strategy[opt]](
+                        start_opts, opt, **kwargs
+                    )
+        except KeyError:
+            message = "Unrecognized strategy: {}. Choose one among: " + str(
+                [strat for strat in self.strategies.keys()]
+            )
+            self.rep.logger.exception(message.format(self.strategy))
+            raise
+        else:
+            return return_opts
+
+    def adaptive(self, start_opts, opt, **kwargs):
+        """Adaptive hyperparameter. Starting with a strong c1, strong w, and weak c2 to increase the
+           exploration of the search space, we want to tend towards a weak c1, weak w, and strong c2 to
+           exploit the best results after exploration by converging towards the global minimum.
+
+           Ref: G. Sermpinis, K. Theofilatos, A. Karathanasopoulos, E. F. Georgopoulos, & C. Dunis,
+           Forecasting foreign exchange rates with adaptive neural networks using radial-basis
+           functions and Particle Swarm Optimization, European Journal of Operational Research.
+            """
+        try:
+            t = kwargs["iternow"]
+            n = kwargs["itermax"]
+            if opt == 'w':
+                new_val = (0.4/n**2) * (t - n)**2 + 0.4
+            elif opt == 'c1':
+                new_val = -3 * t / n + 3.5
+            elif opt == 'c2':
+                new_val = 3 * t / n + 0.5
+            else:
+                self.rep.logger.exception("Invalid opt" +  opt)
+        except KeyError:
+            self.rep.logger.exception("Keyword 'itermax' or 'iternow' missing")
+            raise
+        else:
+            return new_val
+
+    def exp_decay(self, start_opts, opt, **kwargs):
+        """Exponentially decreasing between start and end
+
+        Ref: Li, H.-R., & Gao, Y.-L. (2009). Particle Swarm Optimization Algorithm with Exponent
+        Decreasing Inertia Weight and Stochastic Mutation. 2009 Second International Conference
+        on Information and Computing Science. doi:10.1109/icic.2009.24
+        """
+
+        try:
+            # default values from reference paper
+            if "d1" not in kwargs:
+                d1 = 0.2
+            else:
+                d1 = kwargs["d1"]
+            if "d2" not in kwargs:
+                d2 = 7
+            else:
+                d2 = kwargs["d2"]
+
+            end_opts = {
+                "w": 0.4,
+                "c1": 0.8 * start_opts["c1"],
+                "c2": 1 * start_opts["c2"],
+            }
+            if "end_opts" in kwargs:
+                if opt in kwargs["end_opts"]:
+                    end_opts[opt] = kwargs["end_opts"][opt]
+            start = start_opts[opt]
+            end = end_opts[opt]
+            new_val = (start - end - d1) * math.exp(
+                1 / (1 + d2 * kwargs["iternow"] / kwargs["itermax"])
+            )
+        except KeyError:
+            self.rep.logger.exception("Keyword 'itermax' or 'iternow' missing")
+            raise
+        else:
+            return new_val
+
+    def lin_variation(self, start_opts, opt, **kwargs):
+        """
+        Linearly decreasing/increasing between start and end
+
+        Ref: Shi Y, Eberhart R.: Empirical study of particle swarm optimization Proc
+        of Congress on Computational Intelligence,Washington DC ,USA
+        1999, pp.1945 - 1950.
+        """
+
+        try:
+            end_opts = {
+                "w": 0.4,
+                "c1": 0.8 * start_opts["c1"],
+                "c2": 1 * start_opts["c2"],
+            }
+            if "end_opts" in kwargs:
+                if opt in kwargs["end_opts"]:
+                    end_opts[opt] = kwargs["end_opts"][opt]
+            start = start_opts[opt]
+            end = end_opts[opt]
+            new_val = (
+                end
+                + (start - end)
+                * (kwargs["itermax"] - kwargs["iternow"])
+                / kwargs["itermax"]
+            )
+        except KeyError:
+            self.rep.logger.exception("Keyword 'itermax' or 'iternow' missing")
+            raise
+        else:
+            return new_val
+
+    def random(self, start_opts, opt, **kwargs):
+        """Random value between start option and end option
+
+        Reference: ] R.C. Eberhart, Y.H. Shi, Tracking and optimizing dynamic systems with particle
+        swarms, in: Congress on Evolutionary Computation, Korea, 2001
+        """
+        start = start_opts[opt]
+        if opt in kwargs["end_opts"]:
+            end = kwargs["end_opts"][opt]
+        else:
+            end = start + 1
+        return start + (end - start) * np.random.rand()
+
+    def nonlin_mod(self, start_opts, opt, **kwargs):
+        """Non linear decreasing/increasing with modulation index
+
+        Reference:  A. Chatterjee, P. Siarry, Nonlinear inertia weight variation for dynamic adaption
+        in particle swarm optimization, Computer and Operations Research 33 (2006)
+        859–871, March 2006
+        """
+
+        try:
+            if "n" not in kwargs:
+                n = 1.2
+            else:
+                n = kwargs["n"]
+
+            end_opts = {
+                "w": 0.4,
+                "c1": 0.8 * start_opts["c1"],
+                "c2": 1 * start_opts["c2"],
+            }
+            if "end_opts" in kwargs:
+                if opt in kwargs["end_opts"]:
+                    end_opts[opt] = kwargs["end_opts"][opt]
+
+            start = start_opts[opt]
+            end = end_opts[opt]
+            new_val = end + (start - end) * (
+                (kwargs["itermax"] - kwargs["iternow"]) ** n
+                / kwargs["itermax"] ** n
+            )
+        except KeyError:
+            self.rep.logger.exception("Keyword 'itermax' or 'iternow' missing")
+            raise
+        else:
+            return new_val
